from typing import Optional, Type, Union, cast
from unittest import TestCase

from eppic.commands.argument_parser import ArgumentParser, _unpack_required_type
from eppic.commands.argument_registry import ArgumentRegistry
from eppic.commands.arguments.argument import Argument, ArgumentField
from eppic.input.input_processor import ParsedCommandArgument


class TestArgumentParser(TestCase):
    """Test ArgumentParser class."""

    def setUp(self):
        self.registry = ArgumentRegistry()
        self.registry.register_argument_by_type(int, Argument)
        fields = (ArgumentField("foo", int, True), ArgumentField("bar", int, True),
                  ArgumentField("baz", cast(Type, Optional[int]), False))
        self.parser = ArgumentParser(fields, self.registry)

    def test_unpacking_union(self):
        self.assertEqual(_unpack_required_type(cast(Type, Union[None, str, int])), Union[str, int])

    def test_parse_all_labels(self):
        args = [ParsedCommandArgument(["foo"], '42'), ParsedCommandArgument(["bar"], '0')]
        self.parser.parse_arguments(args)
        self.assertEqual(
            self.parser.get_parsed_values(),
            {"foo": 42, "bar": 0, "baz": None}
        )

    def test_parse_missing_labels(self):
        args = [ParsedCommandArgument(["foo"], '42'), ParsedCommandArgument(None, '0')]
        self.parser.parse_arguments(args)
        self.assertEqual(
            self.parser.get_parsed_values(),
            {"foo": 42, "bar": 0, "baz": None}
        )

    def test_parse_too_many_values(self):
        self.assertRaises(
            ValueError,
            self.parser.parse_arguments, [(["foo"], 42), (["bar"], 0), (["baz"], 0), (None, 3)]
        )

    def test_parse_unknown_label(self):
        self.assertRaises(
            ValueError,
            self.parser.parse_arguments, [(["foo"], 42), (["asdf"], 3)]
        )
